"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ImportFromUrlUseCase = void 0;
var _tasks = require("@webiny/tasks");
var _constants = require("../../../tasks/constants");
var _error = require("@webiny/error");
var _convertTaskToImportRecord = require("../../utils/convertTaskToImportRecord");
class ImportFromUrlUseCase {
  constructor(params) {
    this.updateTask = params.updateTask;
    this.getTask = params.getTask;
    this.triggerTask = params.triggerTask;
  }
  async execute(params) {
    /**
     * First we need to check if the integrity task exists and that it ran successfully.
     */
    const integrityTask = await this.getTask(params.id);
    if (integrityTask?.definitionId !== _constants.VALIDATE_IMPORT_FROM_URL_INTEGRITY_TASK) {
      return null;
    } else if (integrityTask.taskStatus !== _tasks.TaskDataStatus.SUCCESS) {
      throw new _error.WebinyError({
        message: "Integrity check failed.",
        code: "INTEGRITY_CHECK_FAILED",
        data: {
          status: integrityTask.taskStatus,
          files: integrityTask.output?.files,
          error: integrityTask.output?.error
        }
      });
    } else if (!integrityTask.output?.files?.length) {
      throw new _error.WebinyError({
        message: "No files found in the provided data.",
        code: "NO_FILES_FOUND"
      });
    } else if (integrityTask.output.importTaskId) {
      throw new _error.WebinyError({
        message: "Import was already started. You cannot start it again.",
        code: "IMPORT_TASK_EXISTS",
        data: {
          id: integrityTask.output.importTaskId
        }
      });
    }
    const errors = integrityTask.output.files.filter(file => !!file.error);
    if (errors.length) {
      throw new _error.WebinyError({
        message: "Some files failed validation.",
        code: "FILES_FAILED_VALIDATION",
        data: {
          files: errors
        }
      });
    }
    /**
     * Now we need to check if the actual import task already exists.
     * We don't want to run the import task multiple times.
     */
    const importTask = await this.triggerTask({
      name: `Import from URL`,
      definition: _constants.IMPORT_FROM_URL_CONTROLLER_TASK,
      input: {
        modelId: integrityTask.output.modelId,
        files: integrityTask.output.files,
        maxInsertErrors: params.maxInsertErrors,
        steps: {}
      },
      parent: integrityTask
    });
    await this.updateTask(integrityTask.id, {
      output: {
        ...integrityTask.output,
        importTaskId: importTask.id
      }
    });
    return (0, _convertTaskToImportRecord.convertTaskToImportRecord)(importTask);
  }
}
exports.ImportFromUrlUseCase = ImportFromUrlUseCase;

//# sourceMappingURL=ImportFromUrlUseCase.js.map