"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ImportFromUrlDownload = void 0;
var _s3Client = require("../utils/helpers/s3Client");
var _getBucket = require("../utils/helpers/getBucket");
var _upload = require("../utils/upload");
var _importPath = require("../utils/helpers/importPath");
var _downloadFileFromUrl = require("./downloadFileFromUrl");
class ImportFromUrlDownload {
  async run(params) {
    const {
      context,
      response,
      input,
      isCloseToTimeout,
      isAborted
    } = params;
    if (!input.modelId) {
      return response.error({
        message: `Missing "modelId" in the input.`,
        code: "MISSING_MODEL_ID"
      });
    } else if (!input.file) {
      return response.error({
        message: `No file found in the provided data.`,
        code: "NO_FILE_FOUND"
      });
    }
    try {
      await context.cms.getModel(input.modelId);
    } catch (ex) {
      return response.error({
        message: `Model "${input.modelId}" not found.`,
        code: "MODEL_NOT_FOUND"
      });
    }
    const client = (0, _s3Client.createS3Client)();
    const filename = (0, _importPath.prependImportPath)(input.file.key);
    const uploadFactory = (0, _upload.createMultipartUploadFactory)({
      client,
      bucket: (0, _getBucket.getBucket)(),
      filename,
      createHandler: _upload.createMultipartUpload
    });
    const uploadParams = {
      uploadId: input.uploadId
    };
    const upload = await uploadFactory.start(uploadParams);
    const download = (0, _downloadFileFromUrl.createDownloadFileFromUrl)({
      fetch,
      file: {
        url: input.file.get,
        size: input.file.size,
        key: input.file.key
      },
      nextRange: input.nextRange,
      upload
    });
    let result;
    try {
      result = await download.process(async ({
        stop
      }) => {
        const isClose = isCloseToTimeout();
        if (isClose) {
          return stop("continue");
        } else if (isAborted()) {
          return stop("aborted");
        }
      });
    } catch (ex) {
      return response.error(ex);
    }
    switch (result) {
      case "aborted":
        await upload.abort();
        return response.aborted();
      case "continue":
        const continueValue = {
          ...input,
          uploadId: upload.getUploadId(),
          done: download.isDone(),
          nextRange: download.getNextRange()
        };
        return response.continue({
          ...continueValue
        });
      case "done":
        const output = {
          file: filename
        };
        return response.done(output);
      /**
       * There should be nothing else other than "continue" or "aborted" or null.
       */
      default:
        await upload.abort();
        return response.error({
          message: `Method not implemented. Result: ${result}`
        });
    }
  }
}
exports.ImportFromUrlDownload = ImportFromUrlDownload;

//# sourceMappingURL=ImportFromUrlDownload.js.map